package com.garbagemule.MobArena.util.timer;

/**
 * The TimerCallback interface provides hooks for various events in the
 * timer framework. When given a valid callback instance, the timers will
 * call its methods when it starts, stops, finishes and ticks, when
 * applicable.
 */
public interface TimerCallback {
    /**
     * Called when the timer is started.
     * <p>
     * Note that this method is called before the timer is first scheduled,
     * which means the interval can be changed within this method prior to
     * the timer actually starting.
     */
    void onStart();

    /**
     * Called when the timer ticks.
     * <p>
     * Ticks are implementation-specific. Refer to the documentation of the
     * specific timer for details.
     */
    void onTick();

    /**
     * Called when the timer finishes.
     * <p>
     * A timer finishes when it "runs out", which is implementation-specific.
     * For example, the {@link CountdownTimer} finishes when it has counted
     * down to 0.
     */
    void onFinish();

    /**
     * Called when the timer is stopped prematurely.
     * <p>
     * Stopping a timer prematurely is different from the timer naturally
     * running out, however some timers may never run out and must be
     * stopped manually, in which case this method is called instead of
     * the {@code onFinish()} method.
     */
    void onStop();
}
